#!/bin/bash
# We're executing some admin task as root (modifing perms)
ARTIFACTORY_HOME=/var/opt/jfrog/artifactory
ART_ETC_FOLDER=$ARTIFACTORY_HOME/etc

ARTIFACTORY_FOLDERS=$(mount | grep artifactory | awk '{print $3}')

function checkEnv {
    local envVarName=$1
    eval local envVarValue=\$$envVarName
    if [ -z "$envVarValue" ]; then
        echo "Env variable $envVarName is not set, cannot start artifactory instance."
        exit 1
    fi
}

function checkAllEnvs {
    checkEnv ART_LICENSE
    checkEnv ART_DATABASE_CONNECTION_STRING
    checkEnv ART_DATABASE_USER
    checkEnv ART_DATABASE_PASSWORD
}

function setLicense {
    echo -n "$ART_LICENSE" | cut -d, -f1 > /var/opt/jfrog/artifactory/etc/artifactory.lic
    chown artifactory: /var/opt/jfrog/artifactory/etc/artifactory.lic
}

function checkFolderPermissions {
    DIR=$1

    STAT=( $(stat -Lc "%A %G %U" $1) )
    PERM=${STAT[0]}
    GROUP=${STAT[1]}
    USER=${STAT[2]}

    if [[ $USER != "artifactory" ]] || [[ $GROUP != "artifactory"  ]] ; then
            if [[ ! $PERM =~ d......rw.  ]] ; then
                    echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions a+wr"
                    chmod -R a+wr $1
                    chown -R artifactory $1
            fi
    elif [[ $USER == "artifactory" ]] && [[ ! $PERM =~ drw....... ]] ; then
            echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions u+wr"
            chmod -R u+wr $1
    elif [[ $GROUP == "artifactory" ]] && [[ ! $PERM =~ d...rw.... ]] ; then
            echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions g+wr"
            chmod -R g+wr $1
    fi
}

function configureDatabase {
    # Copy the mysql template if the file doesn't exist
    cp -n $ARTIFACTORY_HOME/misc/db/mysql.properties $ARTIFACTORY_HOME/etc/storage.properties
    # Replace url= with provided connection string...
    # Espace sed special chars & \ , 
    # ... before giving it to sed... (yes I know :|)
    local artDatabaseConnectionEscaped=$(echo $ART_DATABASE_CONNECTION_STRING | sed 's,[&\\,],\\&,g')
    local artUserNameEscaped=$(echo $ART_DATABASE_USER | sed 's,[&\\,],\\&,g')
    local artPasswordEscaped=$(echo $ART_DATABASE_PASSWORD | sed 's,[&\\,],\\&,g')
    sed -i -r -e "s,^url=.*$,url=$artDatabaseConnectionEscaped,g" $ARTIFACTORY_HOME/etc/storage.properties
    sed -i -r -e "s,^username=.*$,username=$artUserNameEscaped,g" $ARTIFACTORY_HOME/etc/storage.properties
    sed -i -r -e "s,^password=.*$,password=$artPasswordEscaped,g" $ARTIFACTORY_HOME/etc/storage.properties
    chown artifactory: $ARTIFACTORY_HOME/etc/storage.properties
}

function setPorts {
    # Artifactory's membership port, the PORT0 et PORT1 are specific to MESOS
    : ${MEMBERSHIP_PORT:=10042}
    : ${INSTANCE_PORT:=8081}
    if [ ! -z "$PORT0" ]; then
        INSTANCE_PORT=$PORT0
    fi
    if [ ! -z "$PORT1" ]; then
        MEMBERSHIP_PORT=$PORT1
    fi
    # Change the port
    sed -i -e 's,Connector port="\(.*\)",Connector port="'"$INSTANCE_PORT"'",g' /opt/jfrog/artifactory/tomcat/conf/server.xml
}

function waitForMysql {
   # Extract mysql hostname and port from the jdbc url
    local mysqlHostAndPort=$(echo $ART_DATABASE_CONNECTION_STRING | sed -e 's,^jdbc:mysql:\/\/\(.*\)$,\1:,g' | cut -d/ -f1)
    while ! curl -s "$mysqlHostAndPort" > /dev/null; do echo 'waiting for mysql'; sleep 3; done 
}

function restoreEtcFilesIfEmpty {
    # If etc folder doesn't exists OR exists but default file is missing, copies the default content from $ARTIFACTORY_HOME/defaults
    if [[ ! -d $ART_ETC_FOLDER ]] || [[ ! -e $ART_ETC_FOLDER/default ]] ; then
        echo "[WARN] $ART_ETC_FOLDER is either empty or missing default files, creating default files"
        cp -rpn $ARTIFACTORY_HOME/defaults/etc/* $ART_ETC_FOLDER/
    fi
}

function setInitialConfiguration {
    # Adding Nginx configuration template
    cp -f /tmp/nginx.ftl $ARTIFACTORY_HOME/etc/nginx.ftl
    # Adding licensesBucket plugin
    if [ ! -d $ARTIFACTORY_HOME/etc/plugins ]; then
        mkdir -p $ARTIFACTORY_HOME/etc/plugins
        cp -f /tmp/licensesBucket.groovy $ARTIFACTORY_HOME/etc/plugins/licensesBucket.groovy
        chown -R artifactory:artifactory $ARTIFACTORY_HOME/etc/plugins
    fi
    # Artifactory's external server name
    : ${ART_SERVER_NAME:=artifactory-cluster}
    # Artifactory's port method, default to PORTPERREPO (can be SUBDOMAIN)
    : ${ART_REVERSE_PROXY_METHOD:=portPerRepo}
    # This configuration doesn't exist on the first run
    if [ ! -f $ARTIFACTORY_HOME/etc/artifactory.config.bootstrap.xml ]; then
        sed -i -e "s,\[SERVERNAME\],$ART_SERVER_NAME,g" /tmp/artifactory.config.xml
        sed -i -e "s,\[RPMETHOD\],$ART_REVERSE_PROXY_METHOD,g" /tmp/artifactory.config.xml
        sed -i -e "s,\[PORT\],$INSTANCE_PORT,g" /tmp/artifactory.config.xml        
        mv /tmp/artifactory.config.xml $ARTIFACTORY_HOME/etc/artifactory.config.import.xml
    # On later runs, we'll reimport the latest configuration to change the instance port
    else
        if [ -f $ARTIFACTORY_HOME/etc/artifactory.config.latest.xml ]; then
            cp $ARTIFACTORY_HOME/etc/artifactory.config.latest.xml $ARTIFACTORY_HOME/etc/artifactory.config.import.xml
        else
        # If not we take the bootstrap one, and import it
            cp $ARTIFACTORY_HOME/etc/artifactory.config.bootstrap.xml $ARTIFACTORY_HOME/etc/artifactory.config.import.xml
        fi
        # Changing the instance port
        sed -i -e "s,<artifactoryPort>\(.*\)</artifactoryPort>,<artifactoryPort>$INSTANCE_PORT</artifactoryPort>,g" $ARTIFACTORY_HOME/etc/artifactory.config.import.xml
    fi

    for folder in $ARTIFACTORY_FOLDERS ; do
        checkFolderPermissions $folder
    done
}

function setDefaultJvmParams {
    local defaultFile=$ART_ETC_FOLDER/default
    # Change distrib type
    sed -i s/-Dartdist=rpm/-Dartdist=docker/g $defaultFile
    # In case of marathon, we can set the default Xmx with $MARATHON_APP_RESOURCE_MEM - 250M
    # We let this 250M to allow the non Heap Memory. If this wild guess is not enough, the user
    # can override all the jvm params with $JVM_OPTS
    if [ ! -z "$MARATHON_APP_RESOURCE_MEM" ]; then
        local maxMemory=$(expr $(echo "$MARATHON_APP_RESOURCE_MEM" | cut -d'.' -f 1) - 256)
        local minMemory=$(expr "$maxMemory" / 2)
        echo "[INFO] According to Marathon memory constraint of: $MARATHON_APP_RESOURCE_MEM, will set Xmx to ${maxMemory}m and Xms to ${minMemory}"

        local xmx="-Xms${minMemory}m -Xmx${maxMemory}m"
        local comment="# Adding Xmx"
        # Deleting previous set parameter
        sed -i -e "/^$comment/ { N; d; }" $defaultFile
        # Adding new parameter, don't change the form of this
        printf ''"$comment"' based on Marathon memory constraint \nexport JAVA_OPTIONS="$JAVA_OPTIONS '"$xmx"'"' >> $defaultFile
    fi
    if [ ! -z "$ART_RUNTIME_OPTS" ]; then
        echo "[INFO] Will add defaults JAVA_OPTIONS to $defaultFile with : $ART_RUNTIME_OPTS"
        printf '# Adding user provided options: \nexport JAVA_OPTIONS="$JAVA_OPTIONS '"$ART_RUNTIME_OPTS"'"' >> $defaultFile
    fi
}

checkAllEnvs
setLicense
setPorts
setInitialConfiguration
configureDatabase
restoreEtcFilesIfEmpty
setDefaultJvmParams
# Wait and run as artifactory user
waitForMysql

exec su artifactory -m /opt/jfrog/artifactory/tomcat/bin/catalina.sh run
